"""
Type annotations for cloudformation service client waiters.

[Open documentation](./waiters.md)

Usage::

    ```python
    import boto3

    from mypy_boto3_cloudformation import CloudFormationClient
    from mypy_boto3_cloudformation.waiter import (
        ChangeSetCreateCompleteWaiter,
        StackCreateCompleteWaiter,
        StackDeleteCompleteWaiter,
        StackExistsWaiter,
        StackImportCompleteWaiter,
        StackRollbackCompleteWaiter,
        StackUpdateCompleteWaiter,
        TypeRegistrationCompleteWaiter,
    )

    client: CloudFormationClient = boto3.client("cloudformation")

    change_set_create_complete_waiter: ChangeSetCreateCompleteWaiter = client.get_waiter("change_set_create_complete")
    stack_create_complete_waiter: StackCreateCompleteWaiter = client.get_waiter("stack_create_complete")
    stack_delete_complete_waiter: StackDeleteCompleteWaiter = client.get_waiter("stack_delete_complete")
    stack_exists_waiter: StackExistsWaiter = client.get_waiter("stack_exists")
    stack_import_complete_waiter: StackImportCompleteWaiter = client.get_waiter("stack_import_complete")
    stack_rollback_complete_waiter: StackRollbackCompleteWaiter = client.get_waiter("stack_rollback_complete")
    stack_update_complete_waiter: StackUpdateCompleteWaiter = client.get_waiter("stack_update_complete")
    type_registration_complete_waiter: TypeRegistrationCompleteWaiter = client.get_waiter("type_registration_complete")
    ```
"""
from botocore.waiter import Waiter as Boto3Waiter

from .type_defs import WaiterConfigTypeDef

__all__ = (
    "ChangeSetCreateCompleteWaiter",
    "StackCreateCompleteWaiter",
    "StackDeleteCompleteWaiter",
    "StackExistsWaiter",
    "StackImportCompleteWaiter",
    "StackRollbackCompleteWaiter",
    "StackUpdateCompleteWaiter",
    "TypeRegistrationCompleteWaiter",
)


class ChangeSetCreateCompleteWaiter(Boto3Waiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.change_set_create_complete)[Show boto3-stubs documentation](./waiters.md#changesetcreatecompletewaiter)
    """

    def wait(
        self,
        ChangeSetName: str,
        StackName: str = None,
        NextToken: str = None,
        WaiterConfig: WaiterConfigTypeDef = None,
    ) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.ChangeSetCreateCompleteWaiter)
        [Show boto3-stubs documentation](./waiters.md#changesetcreatecomplete)
        """


class StackCreateCompleteWaiter(Boto3Waiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.stack_create_complete)[Show boto3-stubs documentation](./waiters.md#stackcreatecompletewaiter)
    """

    def wait(
        self, StackName: str = None, NextToken: str = None, WaiterConfig: WaiterConfigTypeDef = None
    ) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.StackCreateCompleteWaiter)
        [Show boto3-stubs documentation](./waiters.md#stackcreatecomplete)
        """


class StackDeleteCompleteWaiter(Boto3Waiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.stack_delete_complete)[Show boto3-stubs documentation](./waiters.md#stackdeletecompletewaiter)
    """

    def wait(
        self, StackName: str = None, NextToken: str = None, WaiterConfig: WaiterConfigTypeDef = None
    ) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.StackDeleteCompleteWaiter)
        [Show boto3-stubs documentation](./waiters.md#stackdeletecomplete)
        """


class StackExistsWaiter(Boto3Waiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.stack_exists)[Show boto3-stubs documentation](./waiters.md#stackexistswaiter)
    """

    def wait(
        self, StackName: str = None, NextToken: str = None, WaiterConfig: WaiterConfigTypeDef = None
    ) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.StackExistsWaiter)
        [Show boto3-stubs documentation](./waiters.md#stackexists)
        """


class StackImportCompleteWaiter(Boto3Waiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.stack_import_complete)[Show boto3-stubs documentation](./waiters.md#stackimportcompletewaiter)
    """

    def wait(
        self, StackName: str = None, NextToken: str = None, WaiterConfig: WaiterConfigTypeDef = None
    ) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.StackImportCompleteWaiter)
        [Show boto3-stubs documentation](./waiters.md#stackimportcomplete)
        """


class StackRollbackCompleteWaiter(Boto3Waiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.stack_rollback_complete)[Show boto3-stubs documentation](./waiters.md#stackrollbackcompletewaiter)
    """

    def wait(
        self, StackName: str = None, NextToken: str = None, WaiterConfig: WaiterConfigTypeDef = None
    ) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.StackRollbackCompleteWaiter)
        [Show boto3-stubs documentation](./waiters.md#stackrollbackcomplete)
        """


class StackUpdateCompleteWaiter(Boto3Waiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.stack_update_complete)[Show boto3-stubs documentation](./waiters.md#stackupdatecompletewaiter)
    """

    def wait(
        self, StackName: str = None, NextToken: str = None, WaiterConfig: WaiterConfigTypeDef = None
    ) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.StackUpdateCompleteWaiter)
        [Show boto3-stubs documentation](./waiters.md#stackupdatecomplete)
        """


class TypeRegistrationCompleteWaiter(Boto3Waiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.type_registration_complete)[Show boto3-stubs documentation](./waiters.md#typeregistrationcompletewaiter)
    """

    def wait(self, RegistrationToken: str, WaiterConfig: WaiterConfigTypeDef = None) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/1.17.87/reference/services/cloudformation.html#CloudFormation.Waiter.TypeRegistrationCompleteWaiter)
        [Show boto3-stubs documentation](./waiters.md#typeregistrationcomplete)
        """
